"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const UpdateChanges_1 = require("../common/UpdateChanges");
const util_1 = require("../common/util");
const version = '13.0.0';
exports.default = () => (host, context) => {
    var _a;
    context.logger.info(`Applying migration for Ignite UI for Angular to version ${version}`);
    const update = new UpdateChanges_1.UpdateChanges(__dirname, host, context);
    const GRIDS = ['IgxGridComponent', 'IgxTreeGridComponent', 'IgxHierarchicalGridComponent'];
    const TAGS = ['igx-grid', 'igx-tree-grid', 'igx-hierarchical-grid'];
    const tsFiles = update.tsFiles;
    for (const path of update.templateFiles) {
        (0, util_1.findElementNodes)((0, util_1.parseFile)(host, path), TAGS)
            .map(node => (0, util_1.getSourceOffset)(node))
            .forEach(offset => {
            const { file, node } = offset;
            if (file.content.includes('columns')) {
                const gridRef = node.attrs.find(e => e.name.includes('#')).name.substring(1);
                const content = file.content.split(gridRef + '.columns').join(gridRef + '.columnsCollection');
                host.overwrite(path, content);
            }
        });
    }
    const getIndicesOf = (searchStr, str) => {
        if (searchStr.length === 0) {
            return [];
        }
        let startIndex = 0;
        let index = 0;
        const indexes = [];
        while ((index = str.indexOf(searchStr, startIndex)) > -1) {
            indexes.push(index);
            startIndex = index + searchStr.length;
        }
        return indexes;
    };
    for (const path of tsFiles) {
        let content = (_a = host.read(path)) === null || _a === void 0 ? void 0 : _a.toString();
        GRIDS.forEach(grid => {
            if (content.indexOf(grid) > -1) {
                const indexes = getIndicesOf('@ViewChild', content);
                indexes.forEach(index => {
                    const viewChildRef = content.substring(index, content.indexOf(';', index)).replace(/\s\s+/g, ' ');
                    if (viewChildRef.includes(grid)) {
                        const gridDeclaration = viewChildRef.substring(viewChildRef.indexOf(')') + 1).replace(/\:/g, '').trim();
                        const gridName = gridDeclaration.split(' ')[1];
                        content = content.split(gridName + '.columns').join(gridName + '.columnsCollection');
                        host.overwrite(path, content);
                    }
                });
            }
        });
    }
    update.applyChanges();
};
